@@ -1,4 +1,4 @@
-# Copyright (C) 2003  Free Software Foundation, Inc.
+# Copyright (C) 2003, 2004  Free Software Foundation, Inc.
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -12,8 +12,8 @@
 
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-# 02111-1307, USA.
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+# 02110-1301, USA.
 
 package Automake::Options;
 
@@ -271,6 +271,27 @@ sub _process_option_list (\%$@)
 	{
 	  # Explicitly recognize these.
 	}
+      elsif ($_ =~ /^filename-length-max=(\d+)$/)
+	{
+	  delete $options->{$_};
+	  $options->{'filename-length-max'} = [$_, $1];
+	}
+      elsif ($_ eq 'tar-v7' || $_ eq 'tar-ustar' || $_ eq 'tar-pax')
+	{
+	  error ($where,
+		 "option `$_' must be an argument of AM_INIT_AUTOMAKE")
+	    if $where->get !~ /^configure\./;
+	  for my $opt ('tar-v7', 'tar-ustar', 'tar-pax')
+	    {
+	      next if $opt eq $_;
+	      if (exists $options->{$opt})
+		{
+		  error ($where,
+			 "options `$_' and `$opt' are mutually exclusive");
+		  last;
+		}
+	    }
+	}
       elsif (/^\d+\.\d+(?:\.\d+)?[a-z]?(?:-[A-Za-z0-9]+)?$/)
 	{
 	  # Got a version number.
